(function ($) {
	var defaults = {
		rowsToDisplay: 10
	};

	var scrollBarWidth = 15, fixedTableWidth;

	$.fn.scrollTableBody = function (options) {
		options = $.extend(defaults, options);

		var table = this;

		wrapTable(table, options);
		alignColumns(table);

		var resizeAlignFunction = function () { alignOnResize(table); };
		$(window).resize(resizeAlignFunction);
	};

	function wrapTable(table, options) {
		var existingClasses = table.attr('class');
		var existingMarginBottom = table.css('margin-bottom');
		table.css('margin-bottom', 0);
		var rowHeight = table.find('tbody tr:first').outerHeight();
		var offset = table.attr("data-scroll-table-offset");
		if (typeof offset === typeof undefined || offset === false) {
			offset = 0;
		}
		var tableHeight = table.parent().innerHeight() - rowHeight - offset;

		var headerTable = $('<table style="table-layout:fixed;width:auto;margin-bottom:0;" class="jqstb-header-table ' + existingClasses + '"><thead><tr><td></td></tr></thead></table>'),
			footerTable = $('<table style="table-layout:fixed;width:auto;margin-bottom:' + existingMarginBottom + ';" class="jqstb-footer-table ' + existingClasses + '"><tfoot><tr><td></td></tr></tfoot></table>'),
			scrollDiv = '<div class="jqstb-scroll" style="height:' + tableHeight + 'px;overflow-y:auto"></div>';

		// Insert the table that will hold the fixed header and footer, and insert the div that will get scrolled
		table.before(headerTable).before(scrollDiv).after(footerTable);
	}

	function alignColumns(table) {
		table.each(function (index) {
			// To minimize "Flash of Unstyled Content" (FOUC), set the relevant variables before manipulating the DOM
			var dataTable = $(this);
			var headerTable = dataTable.prev().prev();
			var footerTable = dataTable.next();

			// Place main table data inside of relevant scrollable div (using jQuery eq() and index)
			var scrollDiv = headerTable.next();
			scrollDiv.prepend(dataTable);
			var scrollEl = scrollDiv[0];

			var hasHorizontalScroll = scrollEl.clientWidth < scrollEl.scrollWidth;
			scrollDiv.outerWidth(fixedTableWidth + scrollBarWidth + 2);

			if (hasHorizontalScroll) {
				var dataTableWidth = dataTable.outerWidth();
				headerTable.width(dataTableWidth);
				footerTable.width(dataTableWidth);
				scrollDiv.outerWidth(scrollEl.clientWidth);
				var scrollDivWidth = scrollDiv.outerWidth();

				var width = scrollDivWidth - scrollBarWidth;

				var headerScrollDiv = $('<div style="overflow:hidden;width:' + width + 'px" class="jqstb-header-scroll"></div>');
				headerTable.wrap(headerScrollDiv);

				var footerScrollDiv = $('<div style="overflow:hidden;width:' + width + 'px" class="jqstb-footer-scroll"></div>');
				footerTable.wrap(footerScrollDiv);

				scrollDiv.on('scroll', function () {
					$('div.jqstb-header-scroll').scrollLeft($(this).scrollLeft());
					$('div.jqstb-footer-scroll').scrollLeft($(this).scrollLeft());
				});
			}

			// Insert header data into fixed header table
			headerTable.find('thead').replaceWith(dataTable.children('caption, thead').clone());

			// Force column widths to be set for each header column
			dataTable.find('tbody tr:first td').each(function (index, value) {
				var target = headerTable.find('thead tr:first th').eq(index);
				target.outerWidth($(this).innerWidth());
			});

			// Force column widths to be set for each footer column
			dataTable.find('tfoot tr:first td').each(function () {
				$(this).outerWidth($(this).outerWidth());
			});

			// Insert footer data into fixed footer table
			footerTable.find('tfoot').replaceWith(dataTable.children('tfoot').clone());

			// Hide original caption, header, and footer
			dataTable.children('caption, thead, tfoot').hide();
		});
	}

	function alignOnResize(table) {
		table.each(function (index) {
			var dataTable = $(this);
			var rowHeight = dataTable.find('tbody tr:first').outerHeight();
			if (rowHeight > 0) {
				var headerTable = dataTable.parent().prev();
				var footerTable = dataTable.parent().next();

				var scrollEl = headerTable.next();
				var hasHorizontalScroll = scrollEl[0].clientWidth < scrollEl[0].scrollWidth;
				if (hasHorizontalScroll) {
					var scrollDivWidth = $('div.jqstb-scroll').outerWidth();
					$('div.jqstb-header-scroll').outerWidth(scrollDivWidth - scrollBarWidth);
					$('div.jqstb-footer-scroll').outerWidth(scrollDivWidth - scrollBarWidth);
				}

				// Force column widths to be set for each header column
				var headerTable = dataTable.parent().prev();
				dataTable.find('tbody tr:first td').each(function (index, value) {
					var target = headerTable.find('thead tr:first th').eq(index);
					target.outerWidth($(this).outerWidth());
				});

				var offset = dataTable.attr("data-scroll-table-offset");
				if (typeof offset === typeof undefined || offset === false) {
					offset = 0;
				}
				var tableHeight = dataTable.parent().parent().innerHeight() - rowHeight - 10 - offset;
				scrollEl.css('height', tableHeight + 'px');
			}
		});
	}
})(jQuery);
